from airflow import macros
from airflow.utils.context import VariableAccessor

from gusty.parsing.loaders import (
    default_constructors,
    handle_user_constructors,
)

airflow_default_macros = {
    "macros": macros,
    "var": {
        "json": VariableAccessor(deserialize_json=True),
        "value": VariableAccessor(deserialize_json=False),
    },
}


def generate_loader_constructors(user_defined_macros, dag_constructors):
    loader_constructors = {}
    loader_constructors.update(default_constructors)
    loader_constructors.update(handle_user_constructors(dag_constructors))

    # user-defined macros are always preferred over
    # any default_constructors or dag_constructors,
    # and have to be formatted for YAML tags
    user_defined_constructors = {}
    for tag, func in user_defined_macros.items():
        tag = "!" + tag
        user_defined_constructors[tag] = func
    loader_constructors.update(user_defined_constructors)

    return loader_constructors


def generate_runner_context(loader_constructors):
    # Assumes that consolidation has already
    # occurred via generate_loader_constructors
    runner_context = {k.strip("!"): v for k, v in loader_constructors.items()}
    # support for airflow_default_macros, as long as the
    # user hasn't already added them via in user_defined_macros
    # or dag_constructors
    for k, v in airflow_default_macros.items():
        if k not in runner_context.keys():
            runner_context[k] = v
    return runner_context


def generate_user_defined_macros(user_defined_macros, dag_constructors):
    macros_plus_constructors = {}
    macros_plus_constructors.update(
        {k.strip("!"): v for k, v in handle_user_constructors(dag_constructors).items()}
    )
    macros_plus_constructors.update(user_defined_macros)
    return macros_plus_constructors



import pytest
from datetime import timedelta
from gusty.utils import days_ago
from gusty import create_dag


# This module is intended to test the default behavior of gusty
# Given a directory of tasks without any metadata located in those directories
# A sort of baseline


##############
## FIXTURES ##
##############


@pytest.fixture(scope="session")
def no_render_on_create_dir():
    return "tests/dags/no_render_on_create"


@pytest.fixture(scope="session")
def dag(no_render_on_create_dir):
    dag = create_dag(
        no_render_on_create_dir,
        description="A dag created without metadata",
        schedule="0 0 * * *",
        default_args={
            "owner": "gusty",
            "depends_on_past": False,
            "start_date": days_ago(1),
            "email": "gusty@gusty.com",
            "email_on_failure": False,
            "email_on_retry": False,
            "retries": 3,
            "retry_delay": timedelta(minutes=5),
        },
    )
    return dag


###########
## TESTS ##
###########


def test_no_render(dag):
    bash_command = dag.task_dict["no_render"].bash_command
    assert bash_command == "echo {{ti.task_id}}"
